#! /usr/bin/python

import commands
import logging
import os
import time
import string
import sys
import threading
import traceback

# for pdb
import pdb
import socket
import sys


log = logging.getLogger("anaconda")
text_screen = None
icw = None
import gettext
_ = lambda x: gettext.ldgettext("anaconda", x)


def logsome(s):
    log.critical("ak: %s", s)

def instance_check(identifier):
    cmd = 'ps ax|grep anaconda|wc'
    output = commands.getoutput(cmd)
    log.critical("%s: %s" % (identifier, output))

def mem_free(identifier):
    cmd = 'cat /proc/meminfo |head -2 |tail -1'
    output = commands.getoutput(cmd)
    log.critical("%s: %s" % (identifier, output))

def debugger(screen = None):
    if screen:
        screen.suspend()
    elif text_screen:
        text_screen.suspend()
    else:
        log.critical("can't suspend screen")
    pdb.set_trace()

def remote_debugger():
    debugger = Rpdb()
    debugger.set_trace(sys._getframe().f_back)

def set_screen(screen):
    global text_screen
    if screen:
        log.critical('setting screen')
        text_screen = screen
    else:
        log.critical('screen is None')

def log_trace(msg = ""):
    stack_strings = traceback.format_stack()
    log.debug("--- ak.log_trace(%s) --" % msg)
    map(log.debug, map(string.strip, stack_strings[:-1]))

def log_trace_exception():
    stack_strings = traceback.format_exc()
    log.debug("--- ak.log_trace_exception() --")
    log.debug(stack_strings)

def give_icw(given_icw):
    global icw
    icw = given_icw

def dimensions(msg):
    (w, h) = icw.window.get_size()
    log.critical("%s: w: %d h: %d" % (msg, w, h))

def gdk_dimensions():
    import gtk
    width = gtk.gdk.screen_width()
    height = gtk.gdk.screen_height()
    log.critical("screen width: %d, screen height: %d", width, height)

def am_i_chrooted(msg):
    pid = os.getpid()
    if os.access('/.profile', os.R_OK):
        log.critical("%s, pid %d, nochroot" % (msg, pid))
    else:
        log.critical("%s, pid %d, CHROOTED!" % (msg, pid))

def dump_modules():
    log.critical("--- ak.dump_modules() --")
    for (key, val) in sys.modules.items():
        log.critical("%s: %s" % (key, val))

def translations():
    log.critical("_Next: %s" % _("_Next"))
    log.critical("Re_view and modify partitioning layout: %s" % _("Re_view and modify partitioning layout"))
    log.critical("_Encrypt system: %s" % _("_Encrypt system"))

def translations2():
    import gtk
    xml = gtk.glade.XML("/tmp/updates/ui/autopart.glade", None, "anaconda")
    button1 = xml.get_widget("encryptButton")
    button2 = xml.get_widget("reviewButton")
    akbutton = xml.get_widget("akbutton")
    log.critical("translations2() %s %s %s"  % (button1.get_label(), button2.get_label(), akbutton.get_label()))
    log.critical("  explicitly: %s %s %s"  % (_(button1.get_label()), _(button2.get_label()), _(akbutton.get_label())))

def dump_all_env():
    log.critical("---dump_env BEGIN---")
    for k in os.environ:
        log.critical("%s: %s" % (k, os.environ[k]))
    log.critical("---dump_env END---")

def dump_env(varname):
    value = os.environ.get(varname, "none!")
    log.critical("value of %s: %s" % (varname, value))

def env_lang():
    log.critical("LANG: %s" % os.environ.get("LANG", "none!"))
    log.critical("LC_MESSAGES: %s" % os.environ.get("LC_MESSAGES", "none!"))

def run_graph_app():
    import iutil;
    s = iutil.execWithCallback("/tmp/updates/run-me.py",[])
    log.critical("after execWithRedirect")
    log.critical(s)

def dialog():
    import locale
    os.environ["LANG"] = 'cs_CZ.UTF-8'
    #os.environ["LANGUAGE"] = 'cs_CZ.UTF-8'
    locale.setlocale(locale.LC_ALL, "")
    env_lang()
    log.critical("gtk gettext: %s" % gettext.ldgettext("gtk20", "(disabled)"))

    gettext.bindtextdomain("gtk20", "/usr/share/locale/")
    gettext.textdomain("gtk20")
    log.critical(gettext.gettext("(disabled)"))

    #  debugger()
    #run_graph_app()
    log.critical("before import: %s COUNT: %d " % (sys.modules.keys, len(sys.modules.keys())))
    import gtk;
    log.critical("after import: %s COUNT: %d " % (sys.modules.keys, len(sys.modules.keys())))

    log.critical('starting dialog')
    dialog = gtk.MessageDialog()
    dialog.add_button("gtk-cancel", 1)
    dialog.run()
    log.critical('finished dialog')

def test_root(storage):
    log.critical("test_root: %s" % storage.rootDevice)

def long_logs():
    strs = 'x' * 1050;
    for i in range(0,50):
        log.critical("%d yeah %s" % (i, strs))

def scdate_gettext():
    import gettext
    try:
        tx = gettext.translation("system-config-date")
        trans = tx.ugettext("America/New_York")
        log.critical("translation: %s" % trans)
    except IOError:
        log.critical("translation: can't has.")

def try_iscsi_fw():
        print '---- starting test.py ----'

        import os
        os.system("modprobe bnx2i")
        os.system("/tmp/updates/brcm_iscsiuio")
        os.system("iscsid")

        import libiscsi
        print libiscsi.say_what()

        nodes = libiscsi.discover_firmware()
        print len(nodes)
        nodes[0].login()

        print '---- finished ----'
        debugger()

def try_udev_devices(msg="try_udev_devices():"):
    from storage.udev import udev_device_is_disk, udev_device_is_md,\
        udev_device_is_dm, udev_device_get_md_container,\
        udev_get_block_devices
    disks = filter(lambda d: udev_device_is_disk(d) and \
                             not udev_device_is_dm(d) and \
                             not udev_device_is_md(d) and \
                             not udev_device_get_md_container(d),
                   udev_get_block_devices())
    log.info(msg + str(disks))

def log_thread(msg=""):
    log.critical("%s %d:%s" % (msg, threading.current_thread().ident,
                               threading.current_thread().name))

"""Remote Python Debugger (pdb wrapper)."""

__author__ = "Bertrand Janin <tamentis@neopulsar.org>"
__version__ = "0.1.1"


class Rpdb(pdb.Pdb):
    def __init__(self, port=4444):
        """Initialize the socket and initialize pdb."""
        addr = socket.gethostname()
        print("pdb is running on %s:%d" % (addr, port))
        log.info("pdb is running on %s:%d" % (addr, port))

        # Backup stdin and stdout before replacing them by the socket handle
        self.old_stdout = sys.stdout
        self.old_stdin = sys.stdin

        # Open a 'reusable' socket to let the webapp reload on the same port
        self.skt = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.skt.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, True)
        self.skt.bind(('0.0.0.0', port))
        self.skt.listen(1)
        (clientsocket, address) = self.skt.accept()
        handle = clientsocket.makefile('rw')
        pdb.Pdb.__init__(self, completekey='tab', stdin=handle, stdout=handle)
        sys.stdout = sys.stdin = handle

    def shutdown(self):
        """Revert stdin and stdout, close the socket."""
        sys.stdout = self.old_stdout
        sys.stdin = self.old_stdin
        self.skt.close()
        self.set_continue()
        print 'pdb shutdown'

    def do_continue(self, arg):
        """Stop all operation on ``continue``."""
        self.shutdown()
        return 1

    do_EOF = do_quit = do_exit = do_c = do_cont = do_continue

def set_trace():
    """Wrapper function to keep the same import x; x.set_trace() interface.

    We catch all the possible exceptions from pdb and cleanup before
    """
    debugger = Rpdb()
    try:
        debugger.set_trace(sys._getframe().f_back)
    finally:
        pass
        #debugger.shutdown()


if __name__ == "__main__":
    for i in range(10):
        print i
        sys.stdout.flush()
        time.sleep(0.3)
